###################################################################################################
# _textformatmanager.py
#
# Implementation of class TextFormatManager (see below).
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###################################################################################################

# Imports.
from Globals import HTMLFile
import DocumentTemplate.DT_HTML
import AccessControl.Role
import string 
import time
import copy
import urllib
# Product Imports.
import _globals 


# ------------------------------------------------------------------------------------------
#  dctFmt:
#
#  List of predefined paragraph-formats. 
# ------------------------------------------------------------------------------------------
dctFmt = [3,'body'
         ,1,'headline 1'
	 ,2,'headline 2'
	 ,9,'headline 3'
	 ,10,'headline 4'
	 ,12,'headline 5'
	 ,13,'headline 6'
	 ,4,'ordered list'
	 ,5,'unordered list'
	 ,6,'indented block'
	 ,7,'emphasis'
	 ,8,'caption'
	 ,11,'plain html'
	]

# ------------------------------------------------------------------------------------------
#  _textformatmanager.textformatrecurse_updateVersion
# ------------------------------------------------------------------------------------------
def textformatrecurse_updateVersion(self, lang, manage_lang, REQUEST):
  msg = ''
  if getattr(self,'build','') < '070':
    for s_preview in [None,'preview']:
      req = {'preview':s_preview}
      ob = self.getObjVersion(req)
      ob_format = getattr(ob,'format',3)
      try:
        delattr(ob,'format')
      except:
        pass
      fmts = dctFmt
      for i in range(0,len(fmts)/2):
        fmt_format = fmts[i*2+0]
        if fmt_format == ob_format:
          fmt_id = fmts[i*2+1]
          fmt_id = string.lower(string.replace(fmt_id,' ','_'))
          setattr(ob,'format',fmt_id)
  return msg

# ------------------------------------------------------------------------------------------
#  _textformatmanager.isManagementInterface
# ------------------------------------------------------------------------------------------
def isManagementInterface(REQUEST):
  return REQUEST is not None and \
         REQUEST.has_key('manage_lang') and \
         string.find(REQUEST.get('URL',''),'/manage_main') > 0 and \
         _globals.isPreviewRequest(REQUEST)
     
# ------------------------------------------------------------------------------------------
#  _textformatmanager.form_quote
# ------------------------------------------------------------------------------------------
def form_quote(text, REQUEST):
  rtn = text
  if isManagementInterface(REQUEST):
    rtn = string.replace(rtn,'<form','<noform')
    rtn = string.replace(rtn,'</form','</noform')
  return rtn
  
# ------------------------------------------------------------------------------------------
#  _textformatmanager.br_quote
# ------------------------------------------------------------------------------------------
def br_quote(text, subtag, REQUEST):
  rtn = ''
  q = ''
  if isManagementInterface(REQUEST):
    if not REQUEST.has_key('format'):
      q = ''
  if subtag == 'br':
    tmp = ""
    for s in string.split(text,'<%s>'%subtag):
      while len(s) > 0 and ord(s[0]) in [10,13]: s = s[1:]
      if len(tmp) > 0:
        tmp = tmp + '\n'
      tmp = tmp + s
    text = tmp
  for s in string.split(text,'\n'):
    subtext = string.strip(s)
    if subtag == 'br':
      rtn = rtn + '\n%s%s<%s>'%(subtext,q,subtag)
    elif len(subtext) > 0:
      rtn = rtn + '\n<%s>%s%s</%s>'%(subtag,subtext,q,subtag)
  return rtn


###################################################################################################
###################################################################################################
###
###   C l a s s   T e x t F o r m a t O b j e c t:
###
###################################################################################################
###################################################################################################
class TextFormatObject:

    # ------------------------------------------------------------------------------------------
    #  TextFormatObject.getTextFormat:
    # ------------------------------------------------------------------------------------------
    def getTextFormat(self, format, REQUEST):
      obs = self.get_text_formats()
      for i in range(0,len(obs)/2):
        id = obs[i*2+0] 
        ob = obs[i*2+1]
        if id == format: return TextFormat(self,id,ob,REQUEST)
      return None

    # ------------------------------------------------------------------------------------------
    #  TextFormatObject.renderText:
    # ------------------------------------------------------------------------------------------
    def renderText(self, id, format, text, REQUEST):
      # Process format.
      textformat = self.getTextFormat(format,REQUEST)
      if textformat is not None:
        text = textformat.renderText(id,text,REQUEST)
      # Process dtml.
      dt_html = DocumentTemplate.DT_HTML.HTML(text)
      text = dt_html(self,REQUEST)
      text = form_quote(text,REQUEST)
      # Return.
      return text

    # ------------------------------------------------------------------------------------------
    #  TextFormatObject.renderPrint:
    # ------------------------------------------------------------------------------------------
    def renderPrint(self, pre, format, text, REQUEST):
      # Process format.
      textformat = self.getTextFormat(format,REQUEST)
      if textformat is not None:
        text = textformat.renderPrint(pre,text,REQUEST)
      # Process dtml.
      dt_html = DocumentTemplate.DT_HTML.HTML(text)
      text = dt_html(self,REQUEST)
      text = form_quote(text,REQUEST)
      # Return.
      return text


###################################################################################################
###################################################################################################
###
###   C l a s s   T e x t F o r m a t M a n a g e r :
###
###################################################################################################
###################################################################################################
class TextFormatManager: 

    # Management Permissions.
    # -----------------------
    __authorPermissions__ = ()
    __editorPermissions__ = ()
    __administratorPermissions__ = (
                'manage_customizeTextFormatForm','manage_customizeTextFormat',
                )
    __ac_permissions__=(
                ('ZMS Administrator', __administratorPermissions__),
                ('ZMS Author', __authorPermissions__),
                ('ZMS Editor', __editorPermissions__),
                )
                

    # Management Interface to customize Text-Formats.
    manage_customizeTextFormatForm = HTMLFile('dtml/textformatmanager/manage_customizeform', globals()) 

    # ------------------------------------------------------------------------------------------
    #  TextFormatManager.__init_text_formats__:
    # ------------------------------------------------------------------------------------------
    def __init_text_formats__(self):
      attr_text_formats = []
      obs = dctFmt
      for i in range(0,len(obs)/2):
        format = obs[i*2+0]
        id = obs[i*2+1]
        lang_attr = 'OPT_F_' + string.upper(string.replace(id,' ',''))
        s_id = string.lower(string.replace(id,' ','_'))
        s_display = {}
        for s_lang in self.getManageLanguages():
          s_display[s_lang] = self.getLangStr(lang_attr,s_lang)
        # Defaults
        s_tag = 'p'
        s_subtag = 'br'
        s_attrs = ''
	# Formats
	if format == 3:
	  pass # s_attrs = 'style="text-align: justify"'
	elif format == 1:
	  s_attrs = 'class="heading1"'
  	elif format == 2:
	  s_attrs = 'class="heading2"'
  	elif format == 9:
	  s_attrs = 'class="heading3"'
  	elif format == 10:
	  s_attrs = 'class="heading4"'
  	elif format == 12:
	  s_attrs = 'class="heading5"'
  	elif format == 13:
	  s_attrs = 'class="heading6"'
  	elif format == 11:
  	  s_tag = ''
  	  s_subtag = ''
  	elif format == 4:
  	  s_tag = 'ol'
  	  s_subtag = 'li'
  	  s_attrs = 'style="numeric"'
  	elif format == 5:
  	  s_tag = 'ul'
  	  s_subtag = 'li'
  	  s_attrs = 'style="bullet"'
  	elif format == 6:
  	  s_tag = 'blockquote'
  	elif format == 7:
	  s_attrs = 'class="emphasis"'
  	elif format == 8:
	  s_attrs = 'class="caption"'
        dct_text_format = {}
        dct_text_format['display'] = s_display
        dct_text_format['tag'] = s_tag
        dct_text_format['subtag'] = s_subtag
        dct_text_format['attrs'] = s_attrs
        attr_text_formats.append(s_id)
        attr_text_formats.append(dct_text_format.copy())
      self.__attr_text_formats__ = copy.deepcopy(attr_text_formats)
      self.__attr_text_formats_default__ = 'body'

    # ------------------------------------------------------------------------------------------
    #  TextFormatManager.get_text_formats:
    # ------------------------------------------------------------------------------------------
    def get_text_formats(self):
      if not hasattr(self,'__attr_text_formats__'):
        self.__init_text_formats__()
      return getattr(self,'__attr_text_formats__',[])

    # ------------------------------------------------------------------------------------------
    #  TextFormatManager.__del_text_format__:
    # ------------------------------------------------------------------------------------------
    def __del_text_format__(self, id):
      attr_text_formats = self.get_text_formats()
      i = attr_text_formats.index(id)
      del attr_text_formats[i]
      del attr_text_formats[i]
      self.__attr_text_formats__ = copy.deepcopy(attr_text_formats)

    # ------------------------------------------------------------------------------------------
    #  TextFormatManager.__set_text_format__:
    # ------------------------------------------------------------------------------------------
    def __set_text_format__(self, old_id, id, display, tag, subtag, attrs, richedit, REQUEST):
      attr_text_formats = self.get_text_formats()
      i = attr_text_formats.index(old_id)
      dct_text_format = attr_text_formats[i+1]
      dct_text_format['display'][REQUEST['manage_lang']] = display
      dct_text_format['tag'] = tag
      dct_text_format['subtag'] = subtag
      dct_text_format['attrs'] = attrs
      dct_text_format['richedit'] = richedit
      attr_text_formats[i+0] = id
      attr_text_formats[i+1] = dct_text_format.copy()
      self.__attr_text_formats__ = copy.deepcopy(attr_text_formats)
    
    # ------------------------------------------------------------------------------------------
    #  TextFormatManager.__add_text_format__:
    # ------------------------------------------------------------------------------------------
    def __add_text_format__(self, id, display, REQUEST):
      if not id in self.getTextFormatIds(REQUEST):
        attr_text_formats = self.get_text_formats()
        dct_text_format = {}
        dct_text_format['display'] = {REQUEST['manage_lang']:display}
        dct_text_format['tag'] = ""
        dct_text_format['subtag'] = ""
        dct_text_format['attrs'] = ""
        dct_text_format['richtedit'] = 0
        attr_text_formats.append(id)
        attr_text_formats.append(dct_text_format.copy())
        self.__attr_text_formats__ = copy.deepcopy(attr_text_formats)
    
    # ------------------------------------------------------------------------------------------
    #  TextFormatManager.__get_text_formats_objs__:
    # ------------------------------------------------------------------------------------------
    def __get_text_formats_objs__(self, REQUEST):
      l_sort = []
      obs = self.get_text_formats()
      for i in range(0,len(obs)/2):
        id = obs[i*2+0] 
        ob = obs[i*2+1]
        textformat = TextFormat(self,id,ob,REQUEST)
        l_sort.append((textformat.getDisplay(),textformat))
      l_sort.sort()
      rtn = []
      for ob in l_sort:
        rtn.append(ob[1])
      return rtn

    # ------------------------------------------------------------------------------------------
    #  TextFormatManager.getTextFormatIds:
    # ------------------------------------------------------------------------------------------
    def getTextFormatIds(self, REQUEST):
      ids = []
      for textformat in self.__get_text_formats_objs__(REQUEST):
        ids.append(textformat.getId())
      return ids

    # ------------------------------------------------------------------------------------------
    #  TextFormatManager.getTextFormats:
    # ------------------------------------------------------------------------------------------
    def getTextFormats(self, REQUEST):
      return self.__get_text_formats_objs__(REQUEST)


    ############################################################################################
    # DEFAULT
    ############################################################################################

    # ------------------------------------------------------------------------------------------
    #  TextFormatManager.getTextFormatDefault:
    # ------------------------------------------------------------------------------------------
    def getTextFormatDefault(self, REQUEST=None):
      dflt = getattr(self,'__attr_text_formats_default__','')
      if len(dflt) == 0 and 'body' in self.getTextFormatIds(REQUEST):
        dflt = 'body'
        self.setTextFormatDefault(dflt)
      return dflt

    # ------------------------------------------------------------------------------------------
    #  TextFormatManager.setTextFormatDefault:
    # ------------------------------------------------------------------------------------------
    def setTextFormatDefault(self, id):
      if len(id) > 0:
        setattr(self,'__attr_text_formats_default__',id)


    ############################################################################################
    # TextFormatManager.manage_customizeTextFormat:
    #
    # Einstellungen fr Absatzformate ndern.
    ############################################################################################
    def manage_customizeTextFormat(self, lang, manage_lang, REQUEST): 
      """ TextFormatManager.manage_customizeTextFormat """
      message = ""

      # Change.
      if REQUEST['btn'] == self.getLangStr('BTN_CHANGE',manage_lang):
        old_id = REQUEST['id']
        id = string.strip(REQUEST['new_id'])
        display = string.strip(REQUEST['new_display'])
        tag = string.strip(REQUEST['new_tag'])
        subtag = string.strip(REQUEST['new_subtag']) 
        attrs = string.strip(REQUEST['new_attrs'])
        richedit = 0
        if REQUEST.has_key('new_richedit'):
          richedit = 1
        self.__set_text_format__(old_id,id,display,tag,subtag,attrs,richedit,REQUEST)
        if REQUEST.has_key('new_default'): self.setTextFormatDefault(REQUEST['new_default'])
        message = urllib.quote(self.getLangStr('MSG_CHANGED',manage_lang))
        
      # Delete.
      elif REQUEST['btn'] == self.getLangStr('BTN_DELETE',manage_lang):
        id = REQUEST['id']
        self.__del_text_format__(id) 
        id = ""
        message = urllib.quote(self.getLangStr('MSG_DELETED',manage_lang)%int(1))
        
      # Insert.
      elif REQUEST['btn'] == self.getLangStr('BTN_INSERT',manage_lang):
        id = string.strip(REQUEST['_id'])
        display = string.strip(REQUEST['_display'])
        self.__add_text_format__(id,display,REQUEST)
        message = urllib.quote(self.getLangStr('MSG_CHANGED',manage_lang))
        
      # Return with message.
      return REQUEST.RESPONSE.redirect('manage_customizeTextFormatForm?lang=%s&manage_lang=%s&manage_tabs_message=%s&id=%s'%(lang,manage_lang,message,id))


###################################################################################################
#  C L A S S   T E X T F O R M A T
###################################################################################################
class TextFormat(AccessControl.Role.RoleManager):

  # Constructor.
  # ------------
  def __init__(self, owner, id, ob, REQUEST):
    self.setOwner(owner)
    self.setId(id)
    if REQUEST is not None and \
       REQUEST.has_key('manage_lang') and \
       ob['display'].has_key(REQUEST['manage_lang']):
      self.setDisplay(ob['display'][REQUEST['manage_lang']])
    else:
      self.setDisplay(id)
    self.setTag(ob['tag'])
    self.setSubTag(ob['subtag'])
    self.setAttrs(ob['attrs'])
    richedit = 0
    if ob.has_key('richedit'):
      richedit = ob['richedit']
    self.setRichedit(richedit)
    

  # Owner
  # -----
  getOwner__roles__ = None # Zope >2.2 SecurityPolicy (Grant unrestricted access!)
  def getOwner(self): return self.owner
  def setOwner(self, owner): self.owner = owner

  # ID
  # --
  getId__roles__ = None # Zope >2.2 SecurityPolicy (Grant unrestricted access!)
  def getId(self): return self.id
  def setId(self, id): self.id = id

  # Display
  # -------
  getDisplay__roles__ = None # Zope >2.2 SecurityPolicy (Grant unrestricted access!)
  def getDisplay(self): return self.display
  def setDisplay(self, display): self.display = display

  # <Tag>
  # -----
  getTag__roles__ = None # Zope >2.2 SecurityPolicy (Grant unrestricted access!)
  def getTag(self): return self.tag
  def setTag(self, tag): self.tag = tag

  # <Sub-Tag>
  # -----------
  getSubTag__roles__ = None # Zope >2.2 SecurityPolicy (Grant unrestricted access!)
  def getSubTag(self): return self.subtag
  def setSubTag(self, subtag): self.subtag = subtag

  # <Tag>-Attributes
  # ----------------
  getAttrs__roles__ = None # Zope >2.2 SecurityPolicy (Grant unrestricted access!)
  def getAttrs(self): return self.attrs
  def setAttrs(self, attrs): self.attrs = attrs

  # Richedit
  # ----------------
  getRichedit__roles__ = None # Zope >2.2 SecurityPolicy (Grant unrestricted access!)
  def getRichedit(self): return self.richedit
  def setRichedit(self, richedit): self.richedit = richedit

  # HTML
  # ----
  getHtml__roles__ = None # Zope >2.2 SecurityPolicy (Grant unrestricted access!)
  def getHtml(self): 
    html = ""
    # Open tag.
    if len(self.getTag()) > 0:
      html = html + "&lt;"
      html = html + self.getTag()
      if len(self.getAttrs()) > 0:
        html = html + " " + self.getAttrs()
      html = html + "&gt;"
      html = html + "<br>"
    # Sub tag.
    subtag = self.getSubTag()
    if len(subtag)>0:
      if subtag == 'br':
        html = html + "&nbsp;&nbsp;...&lt;" + subtag + "&gt;<br>"
      else:
        html = html + "&nbsp;&nbsp;&lt;" + subtag + "&gt;...&lt;/" + subtag + "&gt;<br>"
    else:
      html = html + "..."
    # Close tag.
    if len(self.getTag()) > 0:
      html = html + "&lt;/"
      html = html + self.getTag()
      html = html + "&gt;"
      html = html + "<br>"
    # Default.
    if len(html)==0:
      html = html + "&nbsp;"
    return html

  # Render text
  # -----------
  renderText__roles__ = None # Zope >2.2 SecurityPolicy (Grant unrestricted access!)  
  def renderText(self, id, text, REQUEST):
    html = ""
    # Open tag.
    tag = self.getTag()
    if len(tag) > 0:
      html = html + "<%s"%tag
      if id is not None:
        html = html + ' id="%s"'%id
      if len(self.getAttrs()) > 0:
        html = html + ' ' + self.getAttrs()
      html = html + ">"
    # Sub tag.
    subtag = self.getSubTag()
    if len(subtag) == 0:
      html = html + text
    else:
      html = html + br_quote(text,subtag,REQUEST)
    # Close tag.
    if len(tag) > 0:
      html = html + "</%s>"%tag
    # Default.
    if len(html)==0:
      html = html + "&nbsp;"
    # Return.
    return html
  
  # Render print
  # ------------
  renderPrint__roles__ = None # Zope >2.2 SecurityPolicy (Grant unrestricted access!)  
  def renderPrint(self, pre, text, REQUEST):
    html = ""
    # Open tag.
    tag = self.getTag()
    if len(tag) > 0:
      if tag == 'h1':
        html = html + '<p class="standard"><font size="4" style="font-weight:bold">'
      elif tag == 'h2':
        html = html + '<p class="standard"><font size="4" style="font-weight:bold; font-style:italic;">'
      elif tag == 'h3':
        html = html + '<p class="standard"><font size="3" style="font-weight:bold">'
      elif tag == 'h4':
        html = html + '<p class="standard"><font size="3" style="font-weight:bold; font-style:italic;">'
      elif tag == 'h5':
        html = html + '<p class="standard"><font size="2" style="font-weight:bold">'
      elif tag == 'h6':
        html = html + '<p class="standard"><font size="2" style="font-weight:bold; font-style:italic;">'
      else:
        html = html + "<%s"%tag
        if len(self.getAttrs()) > 0:
          html = html + " " + self.getAttrs()
        html = html + ">"
    # Pre.
    html = html + pre
    # Sub tag.
    subtag = self.getSubTag()
    if len(subtag) == 0:
      html = html + text
    else:
      html = html + br_quote(text,subtag,REQUEST)
    # Close tag.
    if len(tag) > 0:
      if tag in ['h1','h2','h3','h4','h5','h6']:
        html = html + "</font></p>"
      else:
        html = html + "</%s>"%tag
    # Default.
    if len(html)==0:
      html = html + "&nbsp;"
    # Return.
    return html

###################################################################################################
