###################################################################################################
# _zcatalogmanager.py
#
# Implementation of classes ZCatalogItem and ZCatalogManager (see below).
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###################################################################################################

# Imports.
from Products.ZCatalog import ZCatalog, Vocabulary
from App.version_txt import version_txt
import string
import time
import urllib
# Product Imports.
import _globals

# -------------------------------------------------------------------------------------------------
# search_string:
#
# Search string of given value.
# -------------------------------------------------------------------------------------------------
def search_string(v):
  s = ''
  if v is not None:
    if type(v) is type('') and len(v) > 0:
      s = s + v + ' '
    elif type(v) is type([]):
      for i in v:
        s = s + search_string(i)
    elif type(v) is type({}):
      for k in v.keys():
        i = v[k]
        s = s + search_string(i)
  return s

# -------------------------------------------------------------------------------------------------
# search_quote:
#
# Remove HTML-Tags from given string.
# -------------------------------------------------------------------------------------------------
def search_quote(s, maxlen=255, tag="&middot;"):
  # remove all tags.
  stop = 0
  while not stop:
    iStartTag = string.find(s,"<")
    iEndTag = string.find(s,">",iStartTag)
    if iStartTag >= 0 and iEndTag > iStartTag:
      s = s[:iStartTag] + tag + s[iEndTag+1:]
    else:
      stop = 1
  # limit characters.
  if len(s) > maxlen:
    blank = string.find(s,' ',maxlen)
    s = s[:blank] + "&middot;&middot;&middot;"
  # return quoted search string.
  return s

# -------------------------------------------------------------------------------------------------
# _synchronizeCatalogText
# -------------------------------------------------------------------------------------------------
def _synchronizeCatalogText(self, REQUEST):
  el_meta_types = ['ZMS','ZMSRubrik','ZMSDocument','ZMSCustom','ZMSSysFolder']
  for s_lang in self.getLanguages():
    req = {'lang':s_lang,'manage_lang':s_lang}
    zcat_text = ""
    if self.isPage():
      zcat_text = self.catalogText(req)
    setattr(self,'zcat_text_%s'%s_lang,zcat_text)
  if self.isPage():
    for child in self.getChildNodes(REQUEST,el_meta_types):
      _synchronizeCatalogText(child,REQUEST)
        
        
###################################################################################################
###################################################################################################
###
###   class ZCatalogItem
###
###################################################################################################
###################################################################################################
class ZCatalogItem: 

    # ---------------------------------------------------------------------------------------------
    #	ZCatalogItem.search_quote:
    #
    #	Remove HTML-Tags.
    # ---------------------------------------------------------------------------------------------
    def search_quote(self, s, maxlen=255, tag="&middot;"):
      return search_quote(s,maxlen,tag)
    
    # ---------------------------------------------------------------------------------------------
    #	ZMSObject.getCatalogNavUrl:
    #
    #	Returns catalog-navigation url.
    # ---------------------------------------------------------------------------------------------
    def getCatalogNavUrl(self,REQUEST):
      s = REQUEST['URL']
      s = '%s?op=%s'%(s,urllib.quote(REQUEST.get('op','search')))
      s = '%s&raw=%s'%(s,urllib.quote(REQUEST.get('raw','')))
      s = '%s&option=%s'%(s,urllib.quote(REQUEST.get('option','')))
      s = '%s&search_order_by=%s'%(s,urllib.quote(str(REQUEST.get('search_order_by',0))))
      return s

    # ---------------------------------------------------------------------------------------------
    #	ZCatalogItem.catalogText:
    #
    #	Catalog text.
    # ---------------------------------------------------------------------------------------------
    def catalogText(self, REQUEST):
      if _globals.debug: print '[%s.catalogText]: %s'%(self.meta_type,self.absolute_url())
      s = ""
      if self.isVisible(REQUEST):
        for key in self.getObjAttrs().keys():
          obj_attr = self.getObjAttr(key)
          if obj_attr['lang_attr'] and obj_attr['xml']:
            datatype = self.getObjDatatypeKey(obj_attr)
            if datatype in _globals.DT_STRINGS or \
               datatype == _globals.DT_DICTIONARY or \
               datatype == _globals.DT_LIST:
              value = self.getObjProperty(key,REQUEST)
              s = s + search_string(value)
        for ob in self.getChildNodes(REQUEST,self.PAGEELEMENTS):
          s = s + ob.catalogText(REQUEST)
      return s


###################################################################################################
###################################################################################################
###
###   class ZCatalogManager
###
###################################################################################################
###################################################################################################
class ZCatalogManager: 

    # ---------------------------------------------------------------------------------------------
    #	ZCatalogManager.recreateCatalog:
    #
    #	(Re)-creates catalog-object.
    # ---------------------------------------------------------------------------------------------
    def recreateCatalog(self, REQUEST):
      
      ##### Remove ZCatalog ####
      if _globals.debug: print "[%s.recreateCatalog]: Remove ZCatalog..."%(self.meta_type)
      ids = self.objectIds(['ZCatalog'])
      if len(ids) > 0:
        self.manage_delObjects(ids,None)
      
      ##### Add ZCatalog ####
      if _globals.debug: print "[%s.recreateCatalog]: Add ZCatalog..."%(self.meta_type)
      cat_id = 'catalog'
      cat_title = 'Catalog'
      vocab_id = 'create_default_catalog_'
      oCatalog = ZCatalog.ZCatalog(cat_id, cat_title, vocab_id, self)
      self._setObject(oCatalog.id, oCatalog)
      
      ##### Remove Default-Vocabulary ####
      if _globals.debug: print "[%s.recreateCatalog]: Remove Default-Vocabulary..."%(self.meta_type)
      ids = oCatalog.objectIds(['Vocabulary'])
      if len(ids) > 0:
        oCatalog.manage_delObjects(ids,None)
      
      ##### Add ISO 8859-1 Vocabulary ####
      # see ZCatalogs with Umlauts
      # http://www.zope.org/Members/strobl/HowTos/IsoCatalog
      if _globals.debug: print "[%s.recreateCatalog]: Add ISO 8859-1 Vocabulary..."%(self.meta_type)
      id = "Vocabulary"
      title = id
      globbing = 1
      splitter = "ISO_8859_1_Splitter"
      oVocabulary = Vocabulary.Vocabulary(id,title,globbing,splitter)
      oCatalog._setObject(oVocabulary.id,oVocabulary)


    # ---------------------------------------------------------------------------------------------
    #	ZCatalogManager.getCatalog:
    #
    #	Returns catalog-object (None if there is no catalog).
    # ---------------------------------------------------------------------------------------------
    def getCatalog(self):
      return self.getDocumentElement().objectValues(['ZCatalog'])[0]

    # ---------------------------------------------------------------------------------------------
    #	ZCatalogManager.synchronizeSearch:
    # ---------------------------------------------------------------------------------------------
    def synchronizeSearch(self, REQUEST):
      self.setCatalogUpdateFlag()

    # ---------------------------------------------------------------------------------------------
    #	ZCatalogManager.__update_catalog__:
    #
    #	Update (refresh) catalog.
    #	IN:   forced_update	0= not forced (update once a day), 
    #				1= forced (call from management-interface).
    # ---------------------------------------------------------------------------------------------
    def __update_catalog__(self, forced_update, REQUEST):
        
        if forced_update:
          self.recreateCatalog(REQUEST)
          
        do_update = 1
        do_update = do_update and (self.getCatalogUpdateFlag() > 0)
        do_update = do_update and (self.getCatalogPeriod() == '0' or self.getCatalogUpdate())
        do_update = do_update or forced_update
        
        # do update if desired
        if do_update:
          if _globals.debug: print '[%s.__update_catalog__]: do update!'%self.meta_type
            
          _synchronizeCatalogText(self,REQUEST)
            
          oCatalog = self.getCatalog()
          oRoot = self.getDocumentElement().aq_parent
             
          # remove items from catalog
          oCatalog.manage_catalogClear()
                
          # delete indices from catalog
          if version_txt() >= 'Zope 2.5':
            oCatalog.manage_delColumn(oCatalog.schema())
            oCatalog.manage_delIndex(oCatalog.indexes())
          else:
            oCatalog.manage_delColumns(oCatalog.schema())
            oCatalog.manage_deleteIndex(oCatalog.indexes())
                
          # create indices on catalog
          oCatalog.manage_addColumn('getSelf')
          for s_lang in self.getLanguages():
  	    key = 'zcat_text_%s'%s_lang
	    oCatalog.manage_addColumn(key)
  	    oCatalog.manage_addIndex(key,'TextIndex')
  		  
          # find items to catalog
          path = oRoot.absolute_url()[len(str(REQUEST['BASE0']))+1:]
          results = oCatalog.ZopeFindAndApply(oRoot,
                     obj_metatypes=['ZMS','ZMSRubrik','ZMSDocument','ZMSCustom','ZMSSysFolder'],
                     obj_ids=None,
                     obj_searchterm=None,
                     obj_expr=None,
                     obj_mtime=None,
                     obj_mspec=None,
                     obj_permission=None,
                     obj_roles=None,
                     search_sub=1,
                     REQUEST=REQUEST,
                     apply_func=oCatalog.catalog_object,
                     apply_path=path)
                           
          # update time-stamp of last-catalog-update
          self.clearCatalogUpdateFlag()


    # ---------------------------------------------------------------------------------------------
    #	ZCatalogManager.submitCatalogQuery:
    #
    #	Submits query to catalog.
    # ---------------------------------------------------------------------------------------------
    def submitCatalogQuery(self, search_query, search_order_by, search_meta_types, REQUEST):
    
      # Update ZCatalog (0= not forced, update only if necessary)
      self.__update_catalog__(0, REQUEST)
        
      # Search ZCatalog.
      oCatalog = self.getCatalog()
      meta_type = ['ZMS','ZMSRubrik','ZMSDocument','ZMSCustom','ZMSSysFolder']    
      rtn = oCatalog({'meta_type':meta_type,'zcat_text_%s'%REQUEST['lang']:search_query})

      # Sort order.
      if int(search_order_by)==1:
        sOrderBy = 'data_record_score_'
      else:
        sOrderBy = 'change_dt_%s'%REQUEST['lang']
        
      # Sort result-set.
      oResSort = []
      for oItem in rtn:
        oSelf = oItem.getSelf
        if oSelf.isPage() and \
           (len(search_meta_types) == 0 or \
            (oSelf.meta_type != 'ZMSCustom' and oSelf.meta_type in search_meta_types) or \
            (oSelf.meta_type == 'ZMSCustom' and oSelf.meta_id in search_meta_types)):
          sDataRecordId = oItem.data_record_id_
          sPath = REQUEST['BASE0'] + "/" + oCatalog.getpath(sDataRecordId)
          oSummary = {}
          oSummary['title'] = oSelf.getTitle(REQUEST)
          oSummary['content'] = search_quote(getattr(oSelf,'zcat_text_%s'%REQUEST['lang'],''))
          oSummary['url'] = sPath + "/index_%s.html"%REQUEST['lang']
          oSummary['_time'] = oSelf.getObjProperty('change_dt',REQUEST)
          oSummary['_score'] = int(oItem.data_record_score_)
          oResSort.append((getattr(oItem,sOrderBy),oSummary))
      oResSort.sort()
        
      # Return Objects
      rtn = []
      for oItem in oResSort:
        oTmp = [oItem[1]]
        oTmp.extend(rtn)
        rtn = oTmp
      return rtn


    ###############################################################################################
    # Cache Active
    ###############################################################################################

    # ------------------------------------------------------------------------------------------
    # ZCatalogManager.getCatalogPeriod: 
    # ------------------------------------------------------------------------------------------
    def getCatalogPeriod(self):
      return getattr(self,'__catalog_period__','0')

    # ------------------------------------------------------------------------------------------
    # ZCatalogManager.setCatalogPeriod: 
    # ------------------------------------------------------------------------------------------
    def setCatalogPeriod(self, v='1d'):
      self.__catalog_period__ = v


    ###############################################################################################
    # Catalog Update Flag
    ###############################################################################################

    # ------------------------------------------------------------------------------------------
    # ZCatalogManager.clearCatalogUpdateFlag: 
    # ------------------------------------------------------------------------------------------
    def clearCatalogUpdateFlag(self):
      setattr(self,'update_catalog_flag',0)
      setattr(self,'update_catalog_time',time.time())
    
    # ------------------------------------------------------------------------------------------
    # ZCatalogManager.setCatalogUpdateFlag: 
    # ------------------------------------------------------------------------------------------
    def setCatalogUpdateFlag(self):
      setattr(self,'update_catalog_flag',self.getCatalogUpdateFlag()+1)

    # ------------------------------------------------------------------------------------------
    # ZCatalogManager.getCatalogUpdateFlag: 
    # ------------------------------------------------------------------------------------------
    def getCatalogUpdateFlag(self):
      return getattr(self,'update_catalog_flag',0)

    # ------------------------------------------------------------------------------------------
    # ZCatalogManager.getCatalogUpdateTime: 
    # ------------------------------------------------------------------------------------------
    def getCatalogUpdateTime(self):
      return getattr(self,'update_catalog_time',time.time())

    # ------------------------------------------------------------------------------------------
    # ZCatalogManager.getCatalogUpdate: 
    # ------------------------------------------------------------------------------------------
    def getCatalogUpdate(self):
      t_now = time.time()
      t_last_update = getattr(self,'update_catalog_time',time.time())
      return _globals.compareDate(t_now,t_last_update) == 1


    ###############################################################################################
    # ZCatalogManager.manage_updateCatalog:
    #
    # Update (refresh) catalog manually.
    ###############################################################################################
    def manage_updateCatalog(self, lang, manage_lang, REQUEST):
      """ ZMSObject.manage_updateCatalog """
      
      if REQUEST['btn'] in [self.getLangStr('BTN_CHANGE',manage_lang),self.getLangStr('BTN_REFRESH',manage_lang)]:
      
        # Change.
        # -------
        if REQUEST.has_key('period'): 
          self.setCatalogPeriod(REQUEST['period'])
        
        # Update.
        # -------
        if REQUEST['btn'] == self.getLangStr('BTN_REFRESH',manage_lang):
          self.__update_catalog__(1,REQUEST) # Update ZCatalog (1=forced)
          
      # Return with message.
      message = urllib.quote(self.getLangStr('MSG_CHANGED',manage_lang))
      return REQUEST.RESPONSE.redirect('manage_customizeForm?lang=%s&manage_lang=%s&manage_tabs_message=%s#_search'%(lang,manage_lang,message))
      
###################################################################################################
